@@ -3,13 +3,13 @@ from statistics import mean
 import matplotlib.pyplot as plt
 import numpy as np
 from OpenBacktest.ObtUtility import divide, Colors, remove_fees, parse_timestamp
-from OpenBacktest.ObtPositions import PositionBook, LongPosition
+from OpenBacktest.ObtPositions import PositionBook, LongPosition, OrderBook, Order
 
 
-# ----------------------------------------------------------------------------------------------
-# This class represent a wallet and is used to pass buy and sell orders of a symmetric strategy
-# ----------------------------------------------------------------------------------------------
-class Wallet:
+# ------------------------------------------------------------------------------------------------
+# This class represent a symmetric wallet used to pass buy and sell orders of a symmetric strategy
+# ------------------------------------------------------------------------------------------------
+class SymmetricWallet:
     def __init__(self, coin_name, token_name, coin_balance, token_balance, taker, maker, dataframe):
         # Initial coin balance ( will not be modified )
         self.ini_coin_balance = coin_balance
@@ -41,23 +41,13 @@ class Wallet:
         self.total_fees = 0
 
         # data handler
-        self.data_handler = DataHandler(self)
+        self.data_handler = SymmetricDataHandler(self)
 
-    def buy(self, index, amount=None, percent_amount=None):
-
-        if self.position_book.last_position is not None and not self.position_book.last_position.closed:
-            print(Colors.YELLOW, "Advert, buy order is not placeable because a long position is already opened")
+    def buy(self, index):
+        amount = self.coin_balance
+        if amount == 0:
             return
 
-        if amount is None:
-            if percent_amount is None:
-                amount = self.coin_balance
-            else:
-                amount = self.coin_balance * divide(percent_amount, 100)
-        else:
-            if amount > self.coin_balance:
-                amount = self.coin_balance
-
         # Updating Position Book
         # opening a new Position
         position = LongPosition()
@@ -65,23 +55,14 @@ class Wallet:
         self.position_book.add_position(position)
 
         # Buying
-        self.token_balance = remove_fees(divide(amount, self.dataframe["close"][index]), self.taker, self)
+        self.token_balance += remove_fees(divide(amount, self.dataframe["close"][index]), self.taker, self)
         self.coin_balance = self.coin_balance - amount
 
-    def sell(self, index, amount=None, percent_amount=None):
-        if self.position_book.last_position is None or self.position_book.last_position.closed:
-            print(Colors.YELLOW, "Advert, sell order is not placeable because there is no opened long position")
+    def sell(self, index):
+        amount = self.token_balance
+        if amount == 0:
             return
 
-        if amount is None:
-            if percent_amount is None:
-                amount = self.token_balance
-            else:
-                amount = self.token_balance * divide(percent_amount, 100)
-        else:
-            if amount > self.token_balance:
-                amount = self.token_balance
-
         # Selling
         old_coin_balance = self.coin_balance
         self.coin_balance = remove_fees(self.token_balance * self.dataframe["close"][index], self.taker, self)
@@ -94,11 +75,100 @@ class Wallet:
         position.close(self.dataframe["timestamp"][index], self.dataframe["close"][index], self.coin_balance,
                        self.coin_balance - old_coin_balance)
 
+    def get_current_wallet_value(self, index):
+        price = self.dataframe["close"][index]
+        return self.coin_balance + self.token_balance*price
+
+
+# ------------------------------------------------------------------------------------------------------
+# This class represent an asymmetric wallet used to pass buy and sell orders of a an asymetric strategy
+# ------------------------------------------------------------------------------------------------------
+class AsymmetricWallet:
+    def __init__(self, coin_name, token_name, coin_balance, token_balance, taker, maker, dataframe):
+        # Initial coin balance ( will not be modified )
+        self.ini_coin_balance = coin_balance
+        # Initial token balance ( will not be modified )
+        self.ini_token_balance = token_balance
+        # Current coin balance ( should be modified )
+        self.coin_balance = coin_balance
+        # Current token balance
+        self.token_balance = token_balance
+
+        # name of the coin
+        self.coin_name = coin_name
+        # symbol of the token
+        self.token_name = token_name
+
+        # Taker fees
+        self.taker = divide(taker, 100.0)
+        # Maker fees
+        self.maker = divide(maker, 100.0)
+
+        # Represent all orders
+        self.order_book = OrderBook()
+        # Dataframe
+        self.dataframe = dataframe
+
+        # total fees
+        self.total_fees = 0
+
+        # data handler
+        self.data_handler = AsymmetricDataHandler(self)
+
+    def buy(self, index, amount=None, percent_amount=None):
+        if amount is None:
+            if percent_amount is None:
+                amount = self.coin_balance
+            else:
+                amount = self.coin_balance * divide(percent_amount, 100)
+        else:
+            if amount > self.coin_balance:
+                amount = self.coin_balance
+
+        if amount == 0:
+            return
+
+        # Updating Order Book
+        self.order_book.push_order(
+            Order("buy", self.dataframe["timestamp"][index], self.dataframe["close"][index], self.coin_balance,
+                  self.token_balance, amount, None))
+
+        # Buying
+        self.token_balance += remove_fees(divide(amount, self.dataframe["close"][index]), self.taker, self)
+        self.coin_balance -= amount
+
+    def sell(self, index, amount=None, percent_amount=None):
+        if amount is None:
+            if percent_amount is None:
+                amount = self.token_balance
+            else:
+                amount = self.token_balance * divide(percent_amount, 100)
+        else:
+            if amount > self.token_balance:
+                amount = self.token_balance
+
+        if amount == 0:
+            return
+
+        # Selling
+        old_coin_balance = self.coin_balance
+        self.coin_balance += remove_fees(amount * self.dataframe["close"][index], self.taker, self)
+        self.token_balance -= amount
+
+        # Updating Order Book
+        self.order_book.push_order(
+            Order("sell", self.dataframe["timestamp"][index], self.dataframe["close"][index], self.coin_balance,
+                  self.token_balance, None, amount))
+
+    def get_current_wallet_value(self, index):
+        price = self.dataframe["close"][index]
+        return self.coin_balance + self.token_balance*price
+
 
 # -------------------------------------------------------------------------------
 # This class is used to summarize backtest result of a symmetric strategy
 # -------------------------------------------------------------------------------
-class DataHandler:
+class SymmetricDataHandler:
     def __init__(self, wallet):
         self.wallet = wallet
 
@@ -157,6 +227,8 @@ class DataHandler:
         self.average_exposure_time = None
         # average exposure time per trades in %
         self.average_percent_exposure_time = None
+        # drawback / drawdown
+        self.drawdown = 0
 
     # Make the required data to display wallet
     def make_data(self):
@@ -395,11 +467,6 @@ class DataHandler:
 
         # Sell orders timestamps
         sell_timestamps = []
-        # Sell timestamps with the first timestamp
-        sell_timestamps_wallet = [self.start]
-        # Balance at sell orders
-        sell_balance = [self.wallet.ini_coin_balance]
-
         # Buy orders timestamps
         buy_timestamps = []
         # Price at sell orders
@@ -410,18 +477,257 @@ class DataHandler:
         # Append lists
         for trade in self.wallet.position_book.book:
             sell_timestamps.append(trade.sell_timestamp)
-            sell_timestamps_wallet.append(trade.sell_timestamp)
-            sell_balance.append(trade.balance_at_selling)
             buy_timestamps.append(trade.buy_timestamp)
             sell_prices.append(trade.sell_price)
             buy_prices.append(trade.buy_price)
 
         # figure 1, balance
         plt.figure(1)
-        plt.plot(sell_timestamps_wallet, sell_balance)
+        plt.plot(self.wallet.dataframe["timestamp"], self.wallet.dataframe["balance"])
 
         # linear regression
-        x, y = np.array(sell_timestamps_wallet), np.array(sell_balance)
+        x, y = np.array(self.wallet.dataframe["timestamp"]), np.array(self.wallet.dataframe["balance"])
+        m, b = np.polyfit(x, y, 1)
+        plt.plot(x, m * x + b)
+
+        # labels
+        plt.title("Balance")
+        plt.ylabel("balance (" + self.wallet.coin_name + ")")
+        plt.xlabel("Time (timestamps)")
+
+        # figure 2, price
+        if self.wallet.dataframe is not None:
+            plt.figure(2)
+            # main
+            plt.plot(self.wallet.dataframe["timestamp"], self.wallet.dataframe["close"], zorder=1)
+
+            # scatter
+            plt.scatter(buy_timestamps, buy_prices, color="green", marker="^", s=size, zorder=3)
+            plt.scatter(sell_timestamps, sell_prices, color="red", marker="v", s=size, zorder=2)
+
+            # linear regression
+            x, y = np.array(self.wallet.dataframe["timestamp"]), np.array(self.wallet.dataframe["close"])
+            m, b = np.polyfit(x, y, 1)
+            plt.plot(x, m * x + b)
+            # labels
+            plt.title("Price " + self.wallet.token_name + "/" + self.wallet.coin_name)
+            plt.ylabel("price (" + self.wallet.coin_name + ")")
+            plt.xlabel("Time (timestamps)")
+
+        plt.show()
+
+
+# -------------------------------------------------------------------------------
+# This class is used to summarize backtest result of a, asymmetric strategy
+# -------------------------------------------------------------------------------
+class AsymmetricDataHandler:
+    def __init__(self, wallet):
+        self.wallet = wallet
+
+        # Informative values, these values are initialized but will be calculated by make_data() function
+        # coin profit
+        self.profit = 0
+        # coin profit in %
+        self.percent_profit = 0
+        # total number orders
+        self.total_orders = 0
+        # number of buy orders
+        self.total_buy_orders = 0
+        # number of sell orders
+        self.total_sell_orders = 0
+        # Last close time
+        self.end = None
+        # First close time
+        self.start = None
+        # total day of backtest
+        self.total_inday_trading_time = None
+        # first close price
+        self.first_price = None
+        # last close price
+        self.last_price = None
+        # Buy & hold profit from start time to end time
+        self.buy_and_hold_profit = None
+        # Buy & hold profit in % from start time to end time
+        self.buy_and_hold_percent_profit = None
+        # Strategy profit vs buy & hold profit
+        self.strategy_vs_buy_and_hold = None
+        # Strategy profit vs buy & hold profit in %
+        self.strategy_vs_buy_and_hold_percent = None
+        # Average profit per day
+        self.average_profit_per_day = None
+        # aver profit per day in %
+        self.average_percent_profit_per_day = None
+
+    # Make the required data to display wallet
+    def make_data(self):
+        # Total number of trades
+        self.total_orders = len(self.wallet.order_book.book)
+        # return if there were no trades
+        if self.total_orders == 0:
+            return
+
+        # --- Total Profit ---
+        self.profit = self.wallet.coin_balance - self.wallet.ini_coin_balance
+        # Profit in percent depending of initial coin balance
+        self.percent_profit = divide(100 * self.profit, self.wallet.ini_coin_balance)
+
+        # --- Orders ---
+        for order in self.wallet.order_book.book:
+
+            # Count positive & negative trades and collect data to make average positive & negative trades profit
+            if order.type == "buy":
+                self.total_buy_orders += 1
+            elif order.type == "sell":
+                self.total_sell_orders += 1
+
+        # Last timestamp of the period
+        self.end = self.wallet.dataframe['timestamp'][len(self.wallet.dataframe['timestamp']) - 1]
+        # First timestamp of the period
+        self.start = self.wallet.dataframe['timestamp'][0]
+        # Total days from first timestamp to the last one
+        self.total_inday_trading_time = divide(int(self.end - self.start), 86400000)
+        # First close price of the dataframe
+        self.first_price = self.wallet.dataframe['close'][0]
+        # Last close price of the dataframe
+        self.last_price = self.wallet.dataframe['close'][len(self.wallet.dataframe['close']) - 1]
+        # Profit with buy & hold
+        self.buy_and_hold_profit = divide(self.wallet.ini_coin_balance * self.last_price, self.first_price)
+        # Profit with buy & hold in %
+        self.buy_and_hold_percent_profit = divide(100 * self.last_price, self.first_price)
+        # Strategy performance vs buy & hold
+        self.strategy_vs_buy_and_hold = self.profit - self.buy_and_hold_profit
+        # Strategy performance vs buy & hold in %
+        self.strategy_vs_buy_and_hold_percent = self.percent_profit - self.buy_and_hold_percent_profit
+        # Average profit generated per day
+        self.average_profit_per_day = divide(self.profit, self.total_inday_trading_time)
+        # Average profit generated per day in %
+        self.average_percent_profit_per_day = divide(self.percent_profit, self.total_inday_trading_time)
+
+    # Summarize all trades & wallet evolution, strategy stats
+    def display_wallet(self):
+        # Build all required data
+        self.make_data()
+        # If there were no trades
+        if self.total_orders == 0:
+            print(Colors.YELLOW, "This strategy didn't traded")
+            return
+
+        # --- Intro ---
+
+        print(Colors.PURPLE + "----------------------------------------------------")
+        print("Data from", parse_timestamp(self.start), "to", parse_timestamp(self.end))
+        print("Total trading time:", str(round(self.total_inday_trading_time, 2)), "day(s)")
+        print("----------------------------------------------------")
+
+        # --- Wallet data ---
+        # title
+        print(Colors.YELLOW + "[-Wallet-]")
+        # Initial coin balance
+        print(Colors.LIGHT_BLUE, "Initial", self.wallet.coin_name, round(float(self.wallet.ini_coin_balance), 3))
+        # Final coin balance
+        print(Colors.LIGHT_BLUE, "Final", self.wallet.coin_name, round(float(self.wallet.coin_balance), 3))
+        # Initial token balance
+        print(Colors.LIGHT_BLUE, "Initial", self.wallet.token_name, round(float(self.wallet.ini_token_balance), 3))
+        # Final token balance
+        print(Colors.LIGHT_BLUE, "Final", self.wallet.token_name, round(float(self.wallet.token_balance), 6))
+
+        # Profit
+        if self.profit == 0:
+            print(Colors.CYAN, "Strategy profit: ", round(float(self.profit), 2), "/",
+                  str(round(float(self.percent_profit), )) + "%")
+        elif self.profit < 0:
+            print(Colors.RED, "Strategy profit: ", round(float(self.profit), 2), "/",
+                  str(round(float(self.percent_profit), )) + "%")
+        else:
+            print(Colors.GREEN, "Strategy profit: ", round(float(self.profit), 2), "/",
+                  str(round(float(self.percent_profit), )) + "%")
+
+        # Average profit per day
+        print(" Average profit per day: ", round(self.average_profit_per_day, 2), "/",
+              str(round(self.average_percent_profit_per_day, 2)) + "%")
+
+        # Fees
+        print(Colors.LIGHT_RED, "Fees: ", round(float(self.wallet.total_fees), 3), self.wallet.coin_name, "/",
+              str(round(divide(100 * float(self.wallet.total_fees), self.wallet.coin_balance), 2)) + "%")
+
+        # Buy & hold profit
+        if self.buy_and_hold_profit == 0:
+            print(Colors.CYAN, "Buy & hold profit: ", round(float(self.buy_and_hold_profit), 2), "/",
+                  str(round(float(self.buy_and_hold_percent_profit), )) + "%")
+        elif self.buy_and_hold_profit < 0:
+            print(Colors.RED, "Buy & hold profit: ", round(float(self.buy_and_hold_profit), 2), "/",
+                  str(round(float(self.buy_and_hold_percent_profit), )) + "%")
+        else:
+            print(Colors.GREEN, "Buy & hold profit: ", round(float(self.buy_and_hold_profit), 2), "/",
+                  str(round(float(self.buy_and_hold_percent_profit), )) + "%")
+
+        # Strategy vs Buy & Hold
+        if self.strategy_vs_buy_and_hold == 0:
+            print(Colors.CYAN, "Strategy vs buy & hold: ", round(float(self.strategy_vs_buy_and_hold), 2), "/",
+                  str(round(float(self.strategy_vs_buy_and_hold_percent), )) + "%")
+        elif self.strategy_vs_buy_and_hold < 0:
+            print(Colors.RED, "Strategy vs buy & hold: ", round(float(self.strategy_vs_buy_and_hold), 2), "/",
+                  str(round(float(self.strategy_vs_buy_and_hold_percent), )) + "%")
+        else:
+            print(Colors.GREEN, "Strategy vs buy & hold: ", round(float(self.strategy_vs_buy_and_hold), 2), "/",
+                  str(round(float(self.strategy_vs_buy_and_hold_percent), )) + "%")
+
+        print("")
+
+        # --- Orders data ---
+        print(Colors.YELLOW + "[-Orders-]")
+        # Total number of trades
+        print(Colors.BLUE, "Total orders:", self.total_orders)
+        # Total number of positive trades
+        print(Colors.GREEN, "Total buy orders:", self.total_buy_orders)
+        # Total number of negative trades
+        print(Colors.RED, "Total sell orders:", self.total_sell_orders)
+
+    # Make Balance and Token price graphs
+    def plot_wallet(self, size=100):
+        # Total number of trades
+        self.total_orders = len(self.wallet.order_book.book)
+        # Just create a simple graph price if the strategy didn't traded
+        if self.total_orders == 0:
+            # main
+            plt.plot(self.wallet.dataframe["timestamp"], self.wallet.dataframe["close"], zorder=1)
+
+            # linear regression
+            x, y = np.array(self.wallet.dataframe["timestamp"]), np.array(self.wallet.dataframe["close"])
+            m, b = np.polyfit(x, y, 1)
+            plt.plot(x, m * x + b)
+
+            # labels
+            plt.title("Price " + self.wallet.token_name + "/" + self.wallet.coin_name)
+            plt.ylabel("price (" + self.wallet.coin_name + ")")
+            plt.xlabel("Time (timestamps)")
+            plt.show()
+            return
+
+        # Sell orders timestamps
+        sell_timestamps = []
+        # Price at sell orders
+        sell_prices = []
+        # Buy orders timestamps
+        buy_timestamps = []
+        # Price at buy orders
+        buy_prices = []
+
+        # Append lists
+        for order in self.wallet.order_book.book:
+            if order.type == "buy":
+                buy_timestamps.append(order.timestamp)
+                buy_prices.append(order.price)
+            elif order.type == "sell":
+                sell_timestamps.append(order.timestamp)
+                sell_prices.append(order.price)
+
+        # figure 1, balance
+        plt.figure(1)
+        plt.plot(self.wallet.dataframe["timestamp"], self.wallet.dataframe["balance"])
+
+        # linear regression
+        x, y = np.array(self.wallet.dataframe["timestamp"]), np.array(self.wallet.dataframe["balance"])
         m, b = np.polyfit(x, y, 1)
         plt.plot(x, m * x + b)
 
